"""
    Slixmpp: The Slick XMPP Library
    Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
    This file is part of Slixmpp.

    See the file LICENSE for copying permission.
"""

from slixmpp.plugins.base import register_plugin

from slixmpp.plugins.xep_0118 import stanza
from slixmpp.plugins.xep_0118.stanza import UserTune
from slixmpp.plugins.xep_0118.user_tune import XEP_0118


register_plugin(XEP_0118)
