"""
    Slixmpp: The Slick XMPP Library
    Copyright (C) 2010  Nathanael C. Fritz
    This file is part of Slixmpp.

    See the file LICENSE for copying permission.
"""

from slixmpp.roster.item import RosterItem
from slixmpp.roster.single import RosterNode
from slixmpp.roster.multi import Roster
